@@ -74,7 +74,7 @@ int open(const char *path, int flags, mode_t mode)
 		if (mode & O_NONBLOCK) {
 			LD_ISCSI_DPRINTF(0,"Non-blocking I/O is currently not supported");
 			errno = EINVAL;
-			return -1;			
+			return -1;
 		}
 
 		iscsi = iscsi_create_context(initiator);
@@ -92,19 +92,9 @@ int open(const char *path, int flags, mode_t mode)
 			return -1;
 		}
 
-		iscsi_set_targetname(iscsi, iscsi_url->target);
 		iscsi_set_session_type(iscsi, ISCSI_SESSION_NORMAL);
 		iscsi_set_header_digest(iscsi, ISCSI_HEADER_DIGEST_NONE_CRC32C);
 
-		if (iscsi_url->user != NULL) {
-			if (iscsi_set_initiator_username_pwd(iscsi, iscsi_url->user, iscsi_url->passwd) != 0) {
-				LD_ISCSI_DPRINTF(0,"Failed to set initiator username and password");
-				iscsi_destroy_context(iscsi);
-				errno = ENOMEM;
-				return -1;
-			}
-		}
-
 		if (iscsi_full_connect_sync(iscsi, iscsi_url->portal, iscsi_url->lun) != 0) {
 			LD_ISCSI_DPRINTF(0,"Login Failed. %s\n", iscsi_get_error(iscsi));
 			iscsi_destroy_url(iscsi_url);
@@ -131,7 +121,7 @@ int open(const char *path, int flags, mode_t mode)
 			errno = EIO;
 			return -1;
 		}
-      
+
         LD_ISCSI_DPRINTF(4,"readcapacity16_sync: block_size: %d, num_blocks: %"PRIu64,rc16->block_length,rc16->returned_lba + 1);
 
 		fd = iscsi_get_fd(iscsi);
@@ -141,7 +131,7 @@ int open(const char *path, int flags, mode_t mode)
 			iscsi_destroy_context(iscsi);
 			errno = ENFILE;
 			return -1;
-		}		
+		}
 
 		iscsi_fd_list[fd].is_iscsi   = 1;
 		iscsi_fd_list[fd].dup2fd     = -1;
@@ -263,7 +253,7 @@ int __lxstat(int ver, const char *path, struct stat *buf)
 
 		ret = __fxstat(ver, fd, buf);
 		close(fd);
-		return ret;		
+		return ret;
 	}
 
 	return real_lxstat(ver, path, buf);
@@ -303,7 +293,7 @@ off_t lseek(int fd, off_t offset, int whence) {
 		iscsi_fd_list[fd].offset=new_offset;
 		return iscsi_fd_list[fd].offset;
 	}
-	
+
 	return real_lseek(fd, offset, whence);
 }
 
@@ -415,7 +405,7 @@ ssize_t read(int fd, void *buf, size_t count)
 	return real_read(fd, buf, count);
 }
 
-ssize_t (*real_pread)(int fd, void *buf, size_t count, off_t offset); 
+ssize_t (*real_pread)(int fd, void *buf, size_t count, off_t offset);
 ssize_t pread(int fd, void *buf, size_t count, off_t offset) {
 	if ((iscsi_fd_list[fd].is_iscsi == 1 && iscsi_fd_list[fd].in_flight == 0)) {
 		off_t old_offset;
@@ -438,7 +428,7 @@ ssize_t pread(int fd, void *buf, size_t count, off_t offset) {
 
 ssize_t (*real_write)(int fd, const void *buf, size_t count);
 
-ssize_t write(int fd, const void *buf, size_t count) 
+ssize_t write(int fd, const void *buf, size_t count)
 {
 	if ((iscsi_fd_list[fd].is_iscsi == 1) && (iscsi_fd_list[fd].in_flight == 0)) {
 		uint64_t offset;
@@ -485,7 +475,7 @@ ssize_t write(int fd, const void *buf, size_t count)
 
 		iscsi_fd_list[fd].offset += count;
 		scsi_free_scsi_task(task);
-		
+
 		return count;
 	}
 
@@ -543,6 +533,7 @@ int dup2(int oldfd, int newfd)
 	return real_dup2(oldfd, newfd);
 }
 
+#if defined(_LARGEFILE64_SOURCE) && _FILE_OFFSET_BITS != 64
 
 int (*real_fxstat64)(int ver, int fd, struct stat64 *buf);
 
@@ -577,7 +568,7 @@ int __lxstat64(int ver, const char *path, struct stat64 *buf)
 
 		ret = __fxstat64(ver, fd, buf);
 		close(fd);
-		return ret;		
+		return ret;
 	}
 
 	return real_lxstat64(ver, path, buf);
@@ -591,6 +582,7 @@ int __xstat64(int ver, const char *path, struct stat64 *buf)
 	return __lxstat64(ver, path, buf);
 }
 
+#endif
 
 static void __attribute__((constructor)) _init(void)
 {
@@ -669,6 +661,7 @@ static void __attribute__((constructor)) _init(void)
 		exit(10);
 	}
 
+#if defined(_LARGEFILE64_SOURCE) && _FILE_OFFSET_BITS != 64
 	real_fxstat64 = dlsym(RTLD_NEXT, "__fxstat64");
 	if (real_fxstat64 == NULL) {
 		LD_ISCSI_DPRINTF(0,"Failed to dlsym(__fxstat64)");
@@ -683,4 +676,5 @@ static void __attribute__((constructor)) _init(void)
 	if (real_xstat64 == NULL) {
 		LD_ISCSI_DPRINTF(0,"Failed to dlsym(__xstat64)");
 	}
+#endif
 }
