@@ -14,14 +14,23 @@
    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #define _GNU_SOURCE
 
-#if defined(WIN32)
+#if defined(_WIN32)
+#include "win32/win32_compat.h"
 #else
 #include <strings.h>
 #include <unistd.h>
 #endif
 
+#include <assert.h>
+#include <fcntl.h>
+#include <pthread.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -31,8 +40,46 @@
 #include <time.h>
 #include "iscsi.h"
 #include "iscsi-private.h"
+#ifdef HAVE_LINUX_ISER
+#include "iser-private.h"
+#endif
 #include "slist.h"
 
+
+/**
+ * Initialize transport type of session
+ */
+
+int iscsi_init_transport(struct iscsi_context *iscsi,
+			 enum iscsi_transport_type transport) {
+	iscsi->transport = transport;
+
+	switch (iscsi->transport) {
+	case TCP_TRANSPORT:
+		iscsi_init_tcp_transport(iscsi);
+		break;
+#ifdef HAVE_LINUX_ISER
+	case ISER_TRANSPORT:
+		iscsi_init_iser_transport(iscsi);
+		break;
+#endif
+	default:
+		iscsi_set_error(iscsi, "Unfamiliar transport type");
+		return -1;
+	}
+
+	return 0;
+}
+
+/**
+ * Whether or not the internal memory allocator caches allocations. Disable
+ * memory allocation caching to improve the accuracy of Valgrind reports.
+ */
+void iscsi_set_cache_allocations(struct iscsi_context *iscsi, int ca)
+{
+	iscsi->cache_allocations = ca;
+}
+
 void* iscsi_malloc(struct iscsi_context *iscsi, size_t size) {
 	void * ptr = malloc(size);
 	if (ptr != NULL) iscsi->mallocs++;
@@ -68,15 +115,22 @@ char* iscsi_strdup(struct iscsi_context *iscsi, const char* str) {
 	return str2;
 }
 
-void* iscsi_szmalloc(struct iscsi_context *iscsi, size_t size) {
+void* iscsi_smalloc(struct iscsi_context *iscsi, size_t size) {
 	void *ptr;
 	if (size > iscsi->smalloc_size) return NULL;
 	if (iscsi->smalloc_free > 0) {
 		ptr = iscsi->smalloc_ptrs[--iscsi->smalloc_free];
-		memset(ptr, 0, iscsi->smalloc_size);
 		iscsi->smallocs++;
 	} else {
-		ptr = iscsi_zmalloc(iscsi, iscsi->smalloc_size);
+		ptr = iscsi_malloc(iscsi, iscsi->smalloc_size);
+	}
+	return ptr;
+}
+
+void* iscsi_szmalloc(struct iscsi_context *iscsi, size_t size) {
+	void *ptr = iscsi_smalloc(iscsi, size);
+	if (ptr) {
+		memset(ptr, 0, size);
 	}
 	return ptr;
 }
@@ -85,19 +139,63 @@ void iscsi_sfree(struct iscsi_context *iscsi, void* ptr) {
 	if (ptr == NULL) {
 		return;
 	}
-	if (iscsi->smalloc_free == SMALL_ALLOC_MAX_FREE) {
-		int i;
-		/* SMALL_ALLOC_MAX_FREE should be adjusted that this happens rarely */
+	if (!iscsi->cache_allocations) {
+		iscsi_free(iscsi, ptr);
+	} else if (iscsi->smalloc_free == SMALL_ALLOC_MAX_FREE) {
+		/* SMALL_ALLOC_MAX_FREE should be adjusted that this */
+		/* happens rarely */
 		ISCSI_LOG(iscsi, 6, "smalloc free == SMALLOC_MAX_FREE");
-		/* remove oldest half of free pointers and copy
-		 * upper half to lower half */
-		iscsi->smalloc_free >>= 1;
-		for (i = 0; i < iscsi->smalloc_free; i++) {
-			iscsi_free(iscsi, iscsi->smalloc_ptrs[i]);
-			iscsi->smalloc_ptrs[i] = iscsi->smalloc_ptrs[i + iscsi->smalloc_free];
-		}
+		iscsi_free(iscsi, ptr);
+	} else {
+		iscsi->smalloc_ptrs[iscsi->smalloc_free++] = ptr;
 	}
-	iscsi->smalloc_ptrs[iscsi->smalloc_free++] = ptr;
+}
+
+static bool rd_set = false;
+static pthread_mutex_t rd_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+static void
+iscsi_srand_init(struct iscsi_context *iscsi) {
+	unsigned int seed;
+	int urand_fd;
+	ssize_t rc;
+	int err;
+
+	if (rd_set) {
+		/* fast case, seed has been set */
+		return;
+	}
+
+	err = pthread_mutex_lock(&rd_mutex);
+	assert(err == 0);
+
+	if (rd_set) {
+		/* another thread initialized it in the meantime */
+		goto out;
+	}
+
+	urand_fd = open("/dev/urandom", O_RDONLY);
+	if (urand_fd == -1) {
+		goto fallback;
+	}
+
+	rc = read(urand_fd, &seed, sizeof(seed));
+	close(urand_fd);
+	if (rc == -1) {
+		goto fallback;
+	}
+
+	srand(seed);
+	goto out;
+
+fallback:
+	/* seed based on @iscsi */
+	srand(getpid() ^ (uint32_t)((uintptr_t) iscsi));
+
+out:
+	rd_set = true;
+	err = pthread_mutex_unlock(&rd_mutex);
+	assert(err == 0);
 }
 
 struct iscsi_context *
@@ -105,6 +203,7 @@ iscsi_create_context(const char *initiator_name)
 {
 	struct iscsi_context *iscsi;
 	size_t required = ISCSI_RAW_HEADER_SIZE + ISCSI_DIGEST_SIZE;
+	char *ca;
 
 	if (!initiator_name[0]) {
 		return NULL;
@@ -114,16 +213,21 @@ iscsi_create_context(const char *initiator_name)
 	if (iscsi == NULL) {
 		return NULL;
 	}
-	
+
 	memset(iscsi, 0, sizeof(struct iscsi_context));
 
+	/* initalize transport of context */
+	if (iscsi_init_transport(iscsi, TCP_TRANSPORT)) {
+		iscsi_set_error(iscsi, "Failed allocating transport");
+		return NULL;
+	}
+
 	strncpy(iscsi->initiator_name,initiator_name,MAX_STRING_SIZE);
 
 	iscsi->fd = -1;
-	
-	srand(time(NULL) ^ getpid() ^ (uint32_t) ((uintptr_t) iscsi));
 
 	/* initialize to a "random" isid */
+	iscsi_srand_init(iscsi);
 	iscsi_set_isid_random(iscsi, rand(), 0);
 
 	/* assume we start in security negotiation phase */
@@ -179,18 +283,19 @@ iscsi_create_context(const char *initiator_name)
 	/* iscsi->smalloc_size is the size for small allocations. this should be
 	   max(ISCSI_HEADER_SIZE, sizeof(struct iscsi_pdu), sizeof(struct iscsi_in_pdu))
 	   rounded up to the next power of 2. */
+	required = MAX(required, sizeof(struct iscsi_pdu));
+	required = MAX(required, sizeof(struct iscsi_in_pdu));
 	iscsi->smalloc_size = 1;
-	if (sizeof(struct iscsi_pdu) > required) {
-		required = sizeof(struct iscsi_pdu);
-	}
-	if (sizeof(struct iscsi_in_pdu) > required) {
-		required = sizeof(struct iscsi_in_pdu);
-	}
 	while (iscsi->smalloc_size < required) {
 		iscsi->smalloc_size <<= 1;
 	}
 	ISCSI_LOG(iscsi,5,"small allocation size is %d byte", iscsi->smalloc_size);
 
+	ca = getenv("LIBISCSI_CACHE_ALLOCATIONS");
+	if (!ca || atoi(ca) != 0) {
+		iscsi->cache_allocations = 1;
+	}
+
 	return iscsi;
 }
 
@@ -282,7 +387,6 @@ iscsi_set_targetname(struct iscsi_context *iscsi, const char *target_name)
 int
 iscsi_destroy_context(struct iscsi_context *iscsi)
 {
-	struct iscsi_pdu *pdu;
 	int i;
 
 	if (iscsi == NULL) {
@@ -293,41 +397,15 @@ iscsi_destroy_context(struct iscsi_context *iscsi)
 		iscsi_disconnect(iscsi);
 	}
 
-	while ((pdu = iscsi->outqueue)) {
-		ISCSI_LIST_REMOVE(&iscsi->outqueue, pdu);
-		if ( !(pdu->flags & ISCSI_PDU_NO_CALLBACK)) {
-			/* If an error happened during connect/login, we don't want to
-			   call any of the callbacks.
-			 */
-			if (iscsi->is_loggedin) {
-				pdu->callback(iscsi, SCSI_STATUS_CANCELLED, NULL,
-						pdu->private_data);
-			}
-		}
-		iscsi_free_pdu(iscsi, pdu);
-	}
-	while ((pdu = iscsi->waitpdu)) {
-		ISCSI_LIST_REMOVE(&iscsi->waitpdu, pdu);
-		/* If an error happened during connect/login, we don't want to
-		   call any of the callbacks.
-		 */
-		if (iscsi->is_loggedin) {
-			pdu->callback(iscsi, SCSI_STATUS_CANCELLED, NULL,
-					pdu->private_data);
-		}
-		iscsi_free_pdu(iscsi, pdu);
-	}
+	iscsi_cancel_pdus(iscsi);
 
 	if (iscsi->outqueue_current != NULL && iscsi->outqueue_current->flags & ISCSI_PDU_DELETE_WHEN_SENT) {
-		iscsi_free_pdu(iscsi, iscsi->outqueue_current);
+		iscsi->drv->free_pdu(iscsi, iscsi->outqueue_current);
 	}
 
 	if (iscsi->incoming != NULL) {
 		iscsi_free_iscsi_in_pdu(iscsi, iscsi->incoming);
 	}
-	if (iscsi->inqueue != NULL) {
-		iscsi_free_iscsi_inqueue(iscsi, iscsi->inqueue);
-	}
 
 	iscsi->connect_data = NULL;
 
@@ -340,7 +418,14 @@ iscsi_destroy_context(struct iscsi_context *iscsi)
 	} else {
 		ISCSI_LOG(iscsi,5,"memory is clean at iscsi_destroy_context() after %d mallocs, %d realloc(s), %d free(s) and %d reused small allocations",iscsi->mallocs,iscsi->reallocs,iscsi->frees,iscsi->smallocs);
 	}
-	
+
+	if (iscsi->old_iscsi) {
+		iscsi->old_iscsi->fd = -1;
+		iscsi_destroy_context(iscsi->old_iscsi);
+	}
+
+	iscsi_free(iscsi, iscsi->opaque);
+
 	memset(iscsi, 0, sizeof(struct iscsi_context));
 	free(iscsi);
 
@@ -457,12 +542,21 @@ iscsi_parse_url(struct iscsi_context *iscsi, const char *url, int full)
 	char *portal;
 	char *user = NULL;
 	char *passwd = NULL;
+	char *target_user = NULL;
+	char *target_passwd = NULL;
 	char *target = NULL;
 	char *lun;
 	char *tmp;
 	int l = 0;
+#ifdef HAVE_LINUX_ISER
+	int is_iser = 0;
+#endif
 
-	if (strncmp(url, "iscsi://", 8)) {
+	if (strncmp(url, "iscsi://", 8)
+#ifdef HAVE_LINUX_ISER
+            && strncmp(url, "iser://", 7)
+#endif
+            ) {
 		if (full) {
 			iscsi_set_error(iscsi, "Invalid URL %s\niSCSI URL must "
 				"be of the form: %s",
@@ -475,11 +569,61 @@ iscsi_parse_url(struct iscsi_context *iscsi, const char *url, int full)
 		return NULL;
 	}
 
-	strncpy(str,url + 8, MAX_STRING_SIZE);
+#ifdef HAVE_LINUX_ISER
+        if (!strncmp(url, "iser://", 7)) {
+                is_iser = 1;
+                strncpy(str, url + 7, MAX_STRING_SIZE);
+        }
+#endif
+        if (!strncmp(url, "iscsi://", 8)) {
+                strncpy(str, url + 8, MAX_STRING_SIZE);
+        }
 	portal = str;
 
-	user   = getenv("LIBISCSI_CHAP_USERNAME");
-	passwd = getenv("LIBISCSI_CHAP_PASSWORD");
+	user          = getenv("LIBISCSI_CHAP_USERNAME");
+	passwd        = getenv("LIBISCSI_CHAP_PASSWORD");
+	target_user   = getenv("LIBISCSI_CHAP_TARGET_USERNAME");
+	target_passwd = getenv("LIBISCSI_CHAP_TARGET_PASSWORD");
+
+	tmp = strchr(portal, '?');
+	if (tmp) {
+		*tmp++ = 0;
+		while (tmp && *tmp) {
+			char *next = strchr(tmp, '&');
+			char *key, *value;
+			if (next != NULL) {
+				*next++ = 0;
+			}
+			key = tmp;
+			value = strchr(key, '=');
+			if (value != NULL) {
+				*value++ = 0;
+			}
+                        if (!strcmp(key, "header_digest")) {
+                                if (!strcmp(value, "crc32c")) {
+                                        iscsi_set_header_digest(
+                                            iscsi, ISCSI_HEADER_DIGEST_CRC32C);
+                                } else if (!strcmp(value, "none")) {
+                                        iscsi_set_header_digest(
+                                            iscsi, ISCSI_HEADER_DIGEST_NONE);
+                                } else {
+                                        iscsi_set_error(iscsi,
+                                            "Invalid URL argument for header_digest: %s", value);
+                                        return NULL;
+                                }
+                        }
+			if (!strcmp(key, "target_user")) {
+				target_user = value;
+			} else if (!strcmp(key, "target_password")) {
+				target_passwd = value;
+#ifdef HAVE_LINUX_ISER
+			} else if (!strcmp(key, "iser")) {
+				is_iser = 1;
+#endif
+			}
+			tmp = next;
+		}
+	}
 
 	tmp = strchr(portal, '@');
 	if (tmp != NULL) {
@@ -557,11 +701,28 @@ iscsi_parse_url(struct iscsi_context *iscsi, const char *url, int full)
 
 	strncpy(iscsi_url->portal,portal,MAX_STRING_SIZE);
 
-	if (user != NULL && passwd != NULL) {
+	if (user && passwd && user[0] && passwd[0]) {
 		strncpy(iscsi_url->user, user, MAX_STRING_SIZE);
 		strncpy(iscsi_url->passwd, passwd, MAX_STRING_SIZE);
+		/* if we do not have normal CHAP, that means we do not have
+		 * bidirectional either.
+		 */
+		if (target_user && target_passwd && target_user[0] && target_passwd[0]) {
+			strncpy(iscsi_url->target_user, target_user, MAX_STRING_SIZE);
+			strncpy(iscsi_url->target_passwd, target_passwd, MAX_STRING_SIZE);
+		}
 	}
 
+#ifdef HAVE_LINUX_ISER
+	if (iscsi) {
+		if (is_iser) {
+			if (iscsi_init_transport(iscsi, ISER_TRANSPORT))
+				iscsi_set_error(iscsi, "Cannot set transport to iSER");
+		}
+	}
+	iscsi_url->transport = is_iser;
+#endif
+
 	if (full) {
 		strncpy(iscsi_url->target, target, MAX_STRING_SIZE);
 		iscsi_url->lun = l;
@@ -569,6 +730,14 @@ iscsi_parse_url(struct iscsi_context *iscsi, const char *url, int full)
 
 	iscsi_decode_url_string(&iscsi_url->target[0]);
 
+	/* NOTE: iscsi is allowed to be NULL. Especially qemu does call us with iscsi == NULL.
+	 * If we receive iscsi != NULL we apply the parsed settings to the context. */
+	if (iscsi) {
+		iscsi_set_targetname(iscsi, iscsi_url->target);
+		iscsi_set_initiator_username_pwd(iscsi, iscsi_url->user, iscsi_url->passwd);
+		iscsi_set_target_username_pwd(iscsi, iscsi_url->target_user, iscsi_url->target_passwd);
+	}
+
 	return iscsi_url;
 }
 
@@ -600,11 +769,32 @@ int
 iscsi_set_initiator_username_pwd(struct iscsi_context *iscsi,
 						    const char *user, const char *passwd)
 {
+	if (!user || !passwd || !user[0] || !passwd[0]) {
+		iscsi->user[0] = 0;
+		iscsi->passwd[0] = 0;
+		return 0;
+	}
 	strncpy(iscsi->user,user,MAX_STRING_SIZE);
 	strncpy(iscsi->passwd,passwd,MAX_STRING_SIZE);
 	return 0;
 }
 
+
+int
+iscsi_set_target_username_pwd(struct iscsi_context *iscsi,
+			      const char *user, const char *passwd)
+{
+	if (!user || !passwd || !user[0] || !passwd[0]) {
+		iscsi->target_user[0] = 0;
+		iscsi->target_passwd[0] = 0;
+		return 0;
+	}
+	strncpy(iscsi->target_user, user, MAX_STRING_SIZE);
+	strncpy(iscsi->target_passwd, passwd, MAX_STRING_SIZE);
+	return 0;
+}
+
+
 int
 iscsi_set_immediate_data(struct iscsi_context *iscsi, enum iscsi_immediate_data immediate_data)
 {
