@@ -1,4 +1,3 @@
-/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /*
    Copyright (C) 2010 by Ronnie Sahlberg <ronniesahlberg@gmail.com>
 
@@ -27,7 +26,7 @@
 #include <arpa/inet.h>
 #endif
 
-#if defined(WIN32)
+#if defined(_WIN32)
 #include <winsock2.h>
 #endif
 
@@ -2611,6 +2610,52 @@ iscsi_report_supported_opcodes_task(struct iscsi_context *iscsi, int lun,
 	return task;
 }
 
+struct scsi_task *
+iscsi_receive_copy_results_task(struct iscsi_context *iscsi, int lun,
+				int sa, int list_id, int alloc_len,
+				iscsi_command_cb cb, void *private_data)
+{
+	struct scsi_task *task;
+
+	task = scsi_cdb_receive_copy_results(sa, list_id, alloc_len);
+	if (task == NULL) {
+		iscsi_set_error(iscsi, "Out-of-memory: Failed to create "
+				"RECEIVE COPY RESULTS cdb.");
+		return NULL;
+	}
+
+	if (iscsi_scsi_command_async(iscsi, lun, task, cb,
+				     NULL, private_data) != 0) {
+		scsi_free_scsi_task(task);
+		return NULL;
+	}
+
+	return task;
+}
+
+struct scsi_task *
+iscsi_extended_copy_task(struct iscsi_context *iscsi, int lun,
+			 struct iscsi_data *param_data,
+			 iscsi_command_cb cb, void *private_data)
+{
+	struct scsi_task *task;
+
+	task = scsi_cdb_extended_copy(param_data->size);
+	if (task == NULL) {
+		iscsi_set_error(iscsi, "Out-of-memory: Failed to create "
+				"EXTENDED COPY cdb.");
+		return NULL;
+	}
+
+	if (iscsi_scsi_command_async(iscsi, lun, task, cb,
+				     param_data, private_data) != 0) {
+		scsi_free_scsi_task(task);
+		return NULL;
+	}
+
+	return task;
+}
+
 struct scsi_task *
 iscsi_scsi_get_task_from_pdu(struct iscsi_pdu *pdu)
 {
@@ -2651,22 +2696,5 @@ iscsi_scsi_cancel_task(struct iscsi_context *iscsi,
 void
 iscsi_scsi_cancel_all_tasks(struct iscsi_context *iscsi)
 {
-	struct iscsi_pdu *pdu;
-
-	while ((pdu = iscsi->waitpdu)) {
-		ISCSI_LIST_REMOVE(&iscsi->waitpdu, pdu);
-		if (pdu->callback) {
-			pdu->callback(iscsi, SCSI_STATUS_CANCELLED, NULL,
-				      pdu->private_data);
-		}
-		iscsi->drv->free_pdu(iscsi, pdu);
-	}
-	while ((pdu = iscsi->outqueue)) {
-		ISCSI_LIST_REMOVE(&iscsi->outqueue, pdu);
-		if (pdu->callback) {
-			pdu->callback(iscsi, SCSI_STATUS_CANCELLED, NULL,
-				      pdu->private_data);
-		}
-		iscsi->drv->free_pdu(iscsi, pdu);
-	}
+	iscsi_cancel_pdus(iscsi);
 }
