@@ -1,4 +1,3 @@
-/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /*
    Copyright (C) 2010 by Ronnie Sahlberg <ronniesahlberg@gmail.com>
 
@@ -16,7 +15,7 @@
    along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
 
-#if defined(WIN32)
+#if defined(_WIN32)
 #else
 #include <unistd.h>
 #endif
@@ -163,7 +162,7 @@ iscsi_process_nop_out_reply(struct iscsi_context *iscsi, struct iscsi_pdu *pdu,
 	data.data = NULL;
 	data.size = 0;
 
-	if (in->data_pos > ISCSI_HEADER_SIZE) {
+	if (in->data_pos) {
 		data.data = in->data;
 		data.size = in->data_pos;
 	}
