@@ -1,4 +1,3 @@
-/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /*
    Copyright (C) 2010 by Ronnie Sahlberg <ronniesahlberg@gmail.com>
 
@@ -27,8 +26,17 @@
 #include <arpa/inet.h>
 #endif
 
-#if defined(WIN32)
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#else
+#define PRIu64 "llu"
+#define PRIx32 "x"
+#endif
+
+#if defined(_WIN32)
 #include <winsock2.h>
+#include <ws2tcpip.h>
+#include "win32/win32_compat.h"
 #else
 #include <strings.h>
 #endif
@@ -102,7 +110,7 @@ iscsi_allocate_pdu(struct iscsi_context *iscsi, enum iscsi_opcode opcode,
 		return NULL;
 	}
 
-	pdu->outdata.size = ISCSI_HEADER_SIZE;
+	pdu->outdata.size = ISCSI_HEADER_SIZE(iscsi->header_digest);
 	pdu->outdata.data = iscsi_szmalloc(iscsi, pdu->outdata.size);
 
 	if (pdu->outdata.data == NULL) {
@@ -159,7 +167,6 @@ iscsi_tcp_free_pdu(struct iscsi_context *iscsi, struct iscsi_pdu *pdu)
 	iscsi_sfree(iscsi, pdu);
 }
 
-
 int
 iscsi_add_data(struct iscsi_context *iscsi, struct iscsi_data *data,
 	       unsigned char *dptr, int dsize, int pdualignment)
@@ -228,7 +235,7 @@ iscsi_pdu_add_data(struct iscsi_context *iscsi, struct iscsi_pdu *pdu,
 
 	/* update data segment length */
 	scsi_set_uint32(&pdu->outdata.data[4], pdu->outdata.size
-					       - ISCSI_HEADER_SIZE);
+				- ISCSI_HEADER_SIZE(iscsi->header_digest));
 
 	return 0;
 }
@@ -243,7 +250,6 @@ iscsi_get_pdu_data_size(const unsigned char *hdr)
 	return size;
 }
 
-
 int
 iscsi_get_pdu_padding_size(const unsigned char *hdr)
 {
@@ -323,7 +329,6 @@ int iscsi_process_target_nop_in(struct iscsi_context *iscsi,
 	return 0;
 }
 
-
 static void iscsi_reconnect_after_logout(struct iscsi_context *iscsi, int status,
                         void *command_data _U_, void *opaque _U_)
 {
@@ -383,7 +388,6 @@ int iscsi_process_reject(struct iscsi_context *iscsi,
 	return 0;
 }
 
-
 static void iscsi_process_pdu_serials(struct iscsi_context *iscsi, struct iscsi_in_pdu *in)
 {
 	uint32_t itt = scsi_get_uint32(&in->hdr[16]);
@@ -429,6 +433,20 @@ iscsi_process_pdu(struct iscsi_context *iscsi, struct iscsi_in_pdu *in)
 	uint8_t ahslen = in->hdr[4];
 	struct iscsi_pdu *pdu;
 
+	/* verify header checksum */
+	if (iscsi->header_digest != ISCSI_HEADER_DIGEST_NONE) {
+		uint32_t crc, crc_rcvd = 0;
+		crc = crc32c(in->hdr, ISCSI_RAW_HEADER_SIZE);
+		crc_rcvd |= in->hdr[ISCSI_RAW_HEADER_SIZE+0];
+		crc_rcvd |= in->hdr[ISCSI_RAW_HEADER_SIZE+1] << 8;
+		crc_rcvd |= in->hdr[ISCSI_RAW_HEADER_SIZE+2] << 16;
+		crc_rcvd |= in->hdr[ISCSI_RAW_HEADER_SIZE+3] << 24;
+		if (crc != crc_rcvd) {
+			iscsi_set_error(iscsi, "header checksum verification failed: calculated 0x%" PRIx32 " received 0x%" PRIx32, crc, crc_rcvd);
+			return -1;
+		}
+	}
+
 	if (ahslen != 0) {
 		iscsi_set_error(iscsi, "cant handle expanded headers yet");
 		return -1;
@@ -747,5 +765,34 @@ iscsi_timeout_scan(struct iscsi_context *iscsi)
 int
 iscsi_queue_pdu(struct iscsi_context *iscsi, struct iscsi_pdu *pdu)
 {
-        return iscsi->drv->queue_pdu(iscsi, pdu);
+	return iscsi->drv->queue_pdu(iscsi, pdu);
+}
+
+void
+iscsi_cancel_pdus(struct iscsi_context *iscsi)
+{
+	struct iscsi_pdu *pdu;
+
+	while ((pdu = iscsi->outqueue)) {
+		ISCSI_LIST_REMOVE(&iscsi->outqueue, pdu);
+		if (iscsi->is_loggedin && pdu->callback) {
+			/* If an error happened during connect/login,
+			   we don't want to call any of the callbacks.
+			*/
+			pdu->callback(iscsi, SCSI_STATUS_CANCELLED,
+			              NULL, pdu->private_data);
+		}
+		iscsi->drv->free_pdu(iscsi, pdu);
+	}
+	while ((pdu = iscsi->waitpdu)) {
+		ISCSI_LIST_REMOVE(&iscsi->waitpdu, pdu);
+		if (iscsi->is_loggedin && pdu->callback) {
+			/* If an error happened during connect/login,
+			   we don't want to call any of the callbacks.
+			*/
+			pdu->callback(iscsi, SCSI_STATUS_CANCELLED,
+			              NULL, pdu->private_data);
+		}
+		iscsi->drv->free_pdu(iscsi, pdu);
+	}
 }
