@@ -1,4 +1,3 @@
-/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /*
    Copyright (C) 2010 by Ronnie Sahlberg <ronniesahlberg@gmail.com>
 
@@ -39,16 +38,17 @@
 #include "aros/aros_compat.h"
 #endif
 
-#if defined(WIN32)
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+
+#if defined(_WIN32)
 #include <winsock2.h>
 #include <ws2tcpip.h>
 #include "win32/win32_compat.h"
-#define ioctl ioctlsocket
-#define close closesocket
 #else
 #include <strings.h>
 #include <netdb.h>
-#include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 #include <sys/ioctl.h>
@@ -58,7 +58,10 @@
 #include <sys/filio.h>
 #endif
 
+#ifdef HAVE_SYS_UIO_H
 #include <sys/uio.h>
+#endif
+
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -74,6 +77,13 @@
 static uint32_t iface_rr = 0;
 struct iscsi_transport;
 
+/* MUST keep in sync with iser.c */
+union socket_address {
+	struct sockaddr_in sin;
+	struct sockaddr_in6 sin6;
+	struct sockaddr sa;
+};
+
 void
 iscsi_add_to_outqueue(struct iscsi_context *iscsi, struct iscsi_pdu *pdu)
 {
@@ -126,15 +136,15 @@ void iscsi_decrement_iface_rr() {
 	iface_rr--;
 }
 
-static void set_nonblocking(int fd)
+static int set_nonblocking(int fd)
 {
-#if defined(WIN32)
+#if defined(_WIN32)
 	unsigned long opt = 1;
-	ioctlsocket(fd, FIONBIO, &opt);
+	return ioctlsocket(fd, FIONBIO, &opt);
 #else
 	unsigned v;
 	v = fcntl(fd, F_GETFL, 0);
-	fcntl(fd, F_SETFL, v | O_NONBLOCK);
+	return fcntl(fd, F_SETFL, v | O_NONBLOCK);
 #endif
 }
 
@@ -188,6 +198,20 @@ static int iscsi_tcp_connect(struct iscsi_context *iscsi, union socket_address *
 
 	int socksize;
 
+	switch (ai_family) {
+	case AF_INET:
+                socksize = sizeof(struct sockaddr_in);
+                break;
+	case AF_INET6:
+                socksize = sizeof(struct sockaddr_in6);
+                break;
+        default:
+		iscsi_set_error(iscsi, "Unknown address family :%d. "
+				"Only IPv4/IPv6 supported so far.",
+				ai_family);
+                return -1;
+        }
+
 	iscsi->fd = socket(ai_family, SOCK_STREAM, 0);
 	if (iscsi->fd == -1) {
 		iscsi_set_error(iscsi, "Failed to open iscsi socket. "
@@ -203,7 +227,7 @@ static int iscsi_tcp_connect(struct iscsi_context *iscsi, union socket_address *
 		iscsi->fd = iscsi->old_iscsi->fd;
 	}
 
-	set_nonblocking(iscsi->fd);
+	iscsi->tcp_nonblocking = !set_nonblocking(iscsi->fd);
 
 	iscsi_set_tcp_keepalive(iscsi, iscsi->tcp_keepidle, iscsi->tcp_keepcnt, iscsi->tcp_keepintvl);
 
@@ -246,10 +270,11 @@ static int iscsi_tcp_connect(struct iscsi_context *iscsi, union socket_address *
 		ISCSI_LOG(iscsi,3,"TCP_NODELAY set to 1");
 	}
 
-	socksize = sizeof(struct sockaddr_in);  // Work-around for now, need to fix it
-
 	if (connect(iscsi->fd, &sa->sa, socksize) != 0
-		&& errno != EINPROGRESS) {
+#if defined(_WIN32)
+            && WSAGetLastError() != WSAEWOULDBLOCK
+#endif
+            && errno != EINPROGRESS) {
 		iscsi_set_error(iscsi, "Connect failed with errno : "
 			"%s(%d)", strerror(errno), errno);
 		close(iscsi->fd);
@@ -332,6 +357,7 @@ iscsi_connect_async(struct iscsi_context *iscsi, const char *portal,
 	case AF_INET:
 		socksize = sizeof(struct sockaddr_in);
 		memcpy(&sa.sin, ai->ai_addr, socksize);
+                sa.sin.sin_family = AF_INET;
 		sa.sin.sin_port = htons(port);
 #ifdef HAVE_SOCK_SIN_LEN
 		sa.sin.sin_len = socksize;
@@ -341,6 +367,7 @@ iscsi_connect_async(struct iscsi_context *iscsi, const char *portal,
 	case AF_INET6:
 		socksize = sizeof(struct sockaddr_in6);
 		memcpy(&sa.sin6, ai->ai_addr, socksize);
+                sa.sin6.sin6_family = AF_INET6;
 		sa.sin6.sin6_port = htons(port);
 #ifdef HAVE_SOCK_SIN_LEN
 		sa.sin6.sin6_len = socksize;
@@ -360,12 +387,14 @@ iscsi_connect_async(struct iscsi_context *iscsi, const char *portal,
 	iscsi->connect_data      = private_data;
 
 	if (iscsi->drv->connect(iscsi, &sa, ai->ai_family) < 0) {
-		iscsi_set_error(iscsi, "Couldn't connect transport");
+		iscsi_set_error(iscsi, "Couldn't connect transport: %s",
+                                iscsi_get_error(iscsi));
 		freeaddrinfo(ai);
 		return -1;
 	}
 
 	freeaddrinfo(ai);
+	strncpy(iscsi->connected_portal, portal, MAX_STRING_SIZE);
 	return 0;
 }
 
@@ -566,112 +595,130 @@ static int
 iscsi_read_from_socket(struct iscsi_context *iscsi)
 {
 	struct iscsi_in_pdu *in;
-	ssize_t data_size, count, padding_size;
+	ssize_t hdr_size, data_size, count, padding_size;
 
-	if (iscsi->incoming == NULL) {
-		iscsi->incoming = iscsi_szmalloc(iscsi, sizeof(struct iscsi_in_pdu));
-		iscsi->incoming->hdr = iscsi_szmalloc(iscsi, ISCSI_RAW_HEADER_SIZE + ISCSI_DIGEST_SIZE);
+	do {
+		hdr_size = ISCSI_HEADER_SIZE(iscsi->header_digest);
 		if (iscsi->incoming == NULL) {
-			iscsi_set_error(iscsi, "Out-of-memory: failed to malloc iscsi_in_pdu");
-			return -1;
-		}
-	}
-	in = iscsi->incoming;
-
-	/* first we must read the header, including any digests */
-	if (in->hdr_pos < ISCSI_HEADER_SIZE) {
-		/* try to only read the header, the socket is nonblocking, so
-		 * no need to limit the read to what is available in the socket
-		 */
-		count = ISCSI_HEADER_SIZE - in->hdr_pos;
-		count = recv(iscsi->fd, &in->hdr[in->hdr_pos], count, 0);
-		if (count == 0) {
-			return -1;
-		}
-		if (count < 0) {
-			if (errno == EINTR || errno == EAGAIN) {
-				return 0;
+			iscsi->incoming = iscsi_szmalloc(iscsi, sizeof(struct iscsi_in_pdu));
+			if (iscsi->incoming == NULL) {
+				iscsi_set_error(iscsi, "Out-of-memory: failed to malloc iscsi_in_pdu");
+				return -1;
 			}
-			iscsi_set_error(iscsi, "read from socket failed, "
-				"errno:%d", errno);
+			iscsi->incoming->hdr = iscsi_smalloc(iscsi, hdr_size);
+			if (iscsi->incoming->hdr == NULL) {
+				iscsi_set_error(iscsi, "Out-of-memory");
+				return -1;
+			}
+		}
+		in = iscsi->incoming;
+
+		/* first we must read the header, including any digests */
+		if (in->hdr_pos < hdr_size) {
+			/* try to only read the header, the socket is nonblocking, so
+			 * no need to limit the read to what is available in the socket
+			 */
+			count = hdr_size - in->hdr_pos;
+			count = recv(iscsi->fd, &in->hdr[in->hdr_pos], count, 0);
+			if (count == 0) {
+				/* remote side has closed the socket. */
+				return -1;
+			}
+			if (count < 0) {
+				if (errno == EINTR || errno == EAGAIN) {
+					break;
+				}
+				iscsi_set_error(iscsi, "read from socket failed, "
+					"errno:%d", errno);
+				return -1;
+			}
+			in->hdr_pos  += count;
+		}
+
+		if (in->hdr_pos < hdr_size) {
+			/* we don't have the full header yet, so return */
+			break;
+		}
+
+		padding_size = iscsi_get_pdu_padding_size(&in->hdr[0]);
+		data_size = iscsi_get_pdu_data_size(&in->hdr[0]) + padding_size;
+
+		if (data_size < 0 || data_size > (ssize_t)iscsi->initiator_max_recv_data_segment_length) {
+			iscsi_set_error(iscsi, "Invalid data size received from target (%d)", (int)data_size);
 			return -1;
 		}
-		in->hdr_pos  += count;
-	}
+		if (data_size != 0) {
+			unsigned char padding_buf[3];
+			unsigned char *buf = padding_buf;
+			struct scsi_iovector * iovector_in;
 
-	if (in->hdr_pos < ISCSI_HEADER_SIZE) {
-		/* we don't have the full header yet, so return */
-		return 0;
-	}
+			count = data_size - in->data_pos;
 
-	padding_size = iscsi_get_pdu_padding_size(&in->hdr[0]);
-	data_size = iscsi_get_pdu_data_size(&in->hdr[0]) + padding_size;
+			/* first try to see if we already have a user buffer */
+			iovector_in = iscsi_get_scsi_task_iovector_in(iscsi, in);
+			if (iovector_in != NULL && count > padding_size) {
+				uint32_t offset = scsi_get_uint32(&in->hdr[40]);
+				count = iscsi_iovector_readv_writev(iscsi, iovector_in, in->data_pos + offset, count - padding_size, 0);
+			} else {
+				if (iovector_in == NULL) {
+					if (in->data == NULL) {
+						in->data = iscsi_malloc(iscsi, data_size);
+						if (in->data == NULL) {
+							iscsi_set_error(iscsi, "Out-of-memory: failed to malloc iscsi_in_pdu->data(%d)", (int)data_size);
+							return -1;
+						}
+					}
+					buf = &in->data[in->data_pos];
+				}
+				count = recv(iscsi->fd, buf, count, 0);
+			}
+			if (count == 0) {
+				/* remote side has closed the socket. */
+				return -1;
+			}
+			if (count < 0) {
+				if (errno == EINTR || errno == EAGAIN) {
+					break;
+				}
+				iscsi_set_error(iscsi, "read from socket failed, "
+						"errno:%d %s", errno,
+						iscsi_get_error(iscsi));
+				return -1;
+			}
+			in->data_pos += count;
+		}
 
-	if (data_size < 0 || data_size > (ssize_t)iscsi->initiator_max_recv_data_segment_length) {
-		iscsi_set_error(iscsi, "Invalid data size received from target (%d)", (int)data_size);
+		if (in->data_pos < data_size) {
+			break;
+		}
+
+		iscsi->incoming = NULL;
+		if (iscsi_process_pdu(iscsi, in) != 0) {
+			iscsi_free_iscsi_in_pdu(iscsi, in);
+			return -1;
+		}
+		iscsi_free_iscsi_in_pdu(iscsi, in);
+	} while (iscsi->tcp_nonblocking && iscsi->waitpdu && iscsi->is_loggedin);
+
+	return 0;
+}
+
+static int iscsi_pdu_update_headerdigest(struct iscsi_context *iscsi, struct iscsi_pdu *pdu)
+{
+	uint32_t crc;
+
+	if (pdu->outdata.size < ISCSI_RAW_HEADER_SIZE + ISCSI_DIGEST_SIZE) {
+		iscsi_set_error(iscsi, "PDU too small (%u) to contain header digest",
+				(unsigned int) pdu->outdata.size);
 		return -1;
 	}
-	if (data_size != 0) {
-		unsigned char padding_buf[3];
-		unsigned char *buf = padding_buf;
-		struct scsi_iovector * iovector_in;
-
-		count = data_size - in->data_pos;
-
-		/* first try to see if we already have a user buffer */
-		iovector_in = iscsi_get_scsi_task_iovector_in(iscsi, in);
-		if (iovector_in != NULL && count > padding_size) {
-			uint32_t offset = scsi_get_uint32(&in->hdr[40]);
-			count = iscsi_iovector_readv_writev(iscsi, iovector_in, in->data_pos + offset, count - padding_size, 0);
-		} else {
-			if (iovector_in == NULL) {
-				if (in->data == NULL) {
-					in->data = iscsi_malloc(iscsi, data_size);
-					if (in->data == NULL) {
-						iscsi_set_error(iscsi, "Out-of-memory: failed to malloc iscsi_in_pdu->data(%d)", (int)data_size);
-						return -1;
-					}
-				}
-				buf = &in->data[in->data_pos];
-			}
-			count = recv(iscsi->fd, buf, count, 0);
-		}
-		
-		if (count == 0) {
-			return -1;
-		}
-		if (count < 0) {
-			if (errno == EINTR || errno == EAGAIN) {
-				return 0;
-			}
-			iscsi_set_error(iscsi, "read from socket failed, "
-					"errno:%d %s", errno,
-					iscsi_get_error(iscsi));
-			return -1;
-		}
-
-		in->data_pos += count;
-	}
-
-	if (in->data_pos < data_size) {
-		return 0;
-	}
-
-	ISCSI_LIST_ADD_END(&iscsi->inqueue, in);
-	iscsi->incoming = NULL;
-
-
-	while (iscsi->inqueue != NULL) {
-		struct iscsi_in_pdu *current = iscsi->inqueue;
-
-		if (iscsi_process_pdu(iscsi, current) != 0) {
-			return -1;
-		}
-		ISCSI_LIST_REMOVE(&iscsi->inqueue, current);
-		iscsi_free_iscsi_in_pdu(iscsi, current);
-	}
 
+	crc = crc32c(pdu->outdata.data, ISCSI_RAW_HEADER_SIZE);
 
+	pdu->outdata.data[ISCSI_RAW_HEADER_SIZE+3] = (crc >> 24);
+	pdu->outdata.data[ISCSI_RAW_HEADER_SIZE+2] = (crc >> 16);
+	pdu->outdata.data[ISCSI_RAW_HEADER_SIZE+1] = (crc >>  8);
+	pdu->outdata.data[ISCSI_RAW_HEADER_SIZE+0] = (crc);
 	return 0;
 }
 
@@ -724,7 +771,13 @@ iscsi_write_to_socket(struct iscsi_context *iscsi)
 			
 			/* set exp statsn */
 			iscsi_pdu_set_expstatsn(iscsi->outqueue_current, iscsi->statsn + 1);
-			
+
+			/* calculate header checksum */
+			if (iscsi->header_digest != ISCSI_HEADER_DIGEST_NONE &&
+				iscsi_pdu_update_headerdigest(iscsi, iscsi->outqueue_current) != 0) {
+				return -1;
+			}
+
 			ISCSI_LIST_REMOVE(&iscsi->outqueue, iscsi->outqueue_current);
 			if (!(iscsi->outqueue_current->flags & ISCSI_PDU_DELETE_WHEN_SENT)) {
 				/* we have to add the pdu to the waitqueue already here
@@ -954,23 +1007,6 @@ static int iscsi_tcp_queue_pdu(struct iscsi_context *iscsi,
 		return -1;
 	}
 
-	if (iscsi->header_digest != ISCSI_HEADER_DIGEST_NONE) {
-		unsigned long crc;
-
-		if (pdu->outdata.size < ISCSI_RAW_HEADER_SIZE + 4) {
-			iscsi_set_error(iscsi, "PDU too small (%u) to contain header digest",
-					(unsigned int) pdu->outdata.size);
-			return -1;
-		}
-
-		crc = crc32c((char *)pdu->outdata.data, ISCSI_RAW_HEADER_SIZE);
-
-		pdu->outdata.data[ISCSI_RAW_HEADER_SIZE+3] = (crc >> 24)&0xff;
-		pdu->outdata.data[ISCSI_RAW_HEADER_SIZE+2] = (crc >> 16)&0xff;
-		pdu->outdata.data[ISCSI_RAW_HEADER_SIZE+1] = (crc >>  8)&0xff;
-		pdu->outdata.data[ISCSI_RAW_HEADER_SIZE+0] = (crc)      &0xff;
-	}
-
 	iscsi_add_to_outqueue(iscsi, pdu);
 
 	return 0;
@@ -979,23 +1015,13 @@ static int iscsi_tcp_queue_pdu(struct iscsi_context *iscsi,
 void
 iscsi_free_iscsi_in_pdu(struct iscsi_context *iscsi, struct iscsi_in_pdu *in)
 {
-	iscsi_free(iscsi, in->hdr);
+	iscsi_sfree(iscsi, in->hdr);
 	iscsi_free(iscsi, in->data);
 	in->data=NULL;
 	iscsi_sfree(iscsi, in);
 	in=NULL;
 }
 
-void
-iscsi_free_iscsi_inqueue(struct iscsi_context *iscsi, struct iscsi_in_pdu *inqueue)
-{
-	while (inqueue != NULL) {
-	      struct iscsi_in_pdu *next = inqueue->next;
-	      iscsi_free_iscsi_in_pdu(iscsi, inqueue);
-	      inqueue = next;
-	}
-}
-
 void iscsi_set_tcp_syncnt(struct iscsi_context *iscsi, int value)
 {
 	iscsi->tcp_syncnt=value;
@@ -1080,6 +1106,18 @@ void iscsi_set_bind_interfaces(struct iscsi_context *iscsi, char * interfaces _U
 #endif
 }
 
+#if defined(_MSC_VER) && _MSC_VER < 1900
+static iscsi_transport iscsi_transport_tcp = {
+	iscsi_tcp_connect,
+	iscsi_tcp_queue_pdu,
+	iscsi_tcp_new_pdu,
+	iscsi_tcp_disconnect,
+	iscsi_tcp_free_pdu,
+	iscsi_tcp_service,
+	iscsi_tcp_get_fd,
+	iscsi_tcp_which_events,
+};
+#else
 static iscsi_transport iscsi_transport_tcp = {
 	.connect      = iscsi_tcp_connect,
 	.queue_pdu    = iscsi_tcp_queue_pdu,
@@ -1090,6 +1128,7 @@ static iscsi_transport iscsi_transport_tcp = {
 	.get_fd       = iscsi_tcp_get_fd,
 	.which_events = iscsi_tcp_which_events,
 };
+#endif
 
 void iscsi_init_tcp_transport(struct iscsi_context *iscsi)
 {
