@@ -1,4 +1,3 @@
-/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /*
    Copyright (C) 2010 by Ronnie Sahlberg <ronniesahlberg@gmail.com>
 
@@ -27,16 +26,19 @@
 #include "aros/aros_compat.h"
 #endif
 
-#if defined(WIN32)
+#if defined(_WIN32)
 #include <winsock2.h>
 #include "win32/win32_compat.h"
 #endif
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
-#include <unistd.h>
 #include "iscsi.h"
 #include "iscsi-private.h"
 #include "scsi-lowlevel.h"
@@ -85,10 +87,8 @@ iscsi_sync_cb(struct iscsi_context *iscsi _U_, int status,
 {
 	struct iscsi_sync_state *state = private_data;
 
-	if (state != NULL) {
-		state->status    = status;
-		state->finished = 1;
-	}
+	state->status    = status;
+	state->finished = 1;
 }
 
 int
@@ -111,7 +111,12 @@ iscsi_connect_sync(struct iscsi_context *iscsi, const char *portal)
 	/* clear connect_data so it doesnt point to our stack */
 	iscsi->connect_data = NULL;
 
-	return state.status;
+	/* in case of error, cancel any pending pdus */
+	if (state.status != SCSI_STATUS_GOOD) {
+		iscsi_cancel_pdus(iscsi);
+	}
+
+	return (state.status == SCSI_STATUS_GOOD) ? 0 : -1;
 }
 
 int
@@ -132,7 +137,12 @@ iscsi_full_connect_sync(struct iscsi_context *iscsi,
 
 	event_loop(iscsi, &state);
 
-	return state.status;
+	/* in case of error, cancel any pending pdus */
+	if (state.status != SCSI_STATUS_GOOD) {
+		iscsi_cancel_pdus(iscsi);
+	}
+
+	return (state.status == SCSI_STATUS_GOOD) ? 0 : -1;
 }
 
 int iscsi_login_sync(struct iscsi_context *iscsi)
@@ -149,7 +159,7 @@ int iscsi_login_sync(struct iscsi_context *iscsi)
 
 	event_loop(iscsi, &state);
 
-	return state.status;
+	return (state.status == SCSI_STATUS_GOOD) ? 0 : -1;
 }
 
 int iscsi_logout_sync(struct iscsi_context *iscsi)
@@ -166,7 +176,7 @@ int iscsi_logout_sync(struct iscsi_context *iscsi)
 
 	event_loop(iscsi, &state);
 
-	return state.status;
+	return (state.status == SCSI_STATUS_GOOD) ? 0 : -1;
 }
 
 static void
@@ -213,7 +223,7 @@ int iscsi_reconnect_sync(struct iscsi_context *iscsi)
 
 	reconnect_event_loop(iscsi, &state);
 
-	return state.status;
+	return (state.status == SCSI_STATUS_GOOD) ? 0 : -1;
 }
 
 static void
@@ -270,7 +280,7 @@ iscsi_task_mgmt_sync(struct iscsi_context *iscsi,
 
 	event_loop(iscsi, &state);
 
-	return state.status;
+	return (state.status == SCSI_STATUS_GOOD) ? 0 : -1;
 }
 
 int
@@ -1667,6 +1677,46 @@ iscsi_report_supported_opcodes_sync(struct iscsi_context *iscsi, int lun,
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_receive_copy_results_sync(struct iscsi_context *iscsi, int lun,
+				int sa, int list_id, int alloc_len)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_receive_copy_results_task(iscsi, lun, sa, list_id, alloc_len,
+					    scsi_sync_cb, &state) == NULL) {
+		iscsi_set_error(iscsi, "Failed to send RECEIVE COPY RESULTS"
+				" command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
+struct scsi_task *
+iscsi_extended_copy_sync(struct iscsi_context *iscsi, int lun,
+			 struct iscsi_data *param_data)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_extended_copy_task(iscsi, lun, param_data,
+					scsi_sync_cb, &state) == NULL) {
+		iscsi_set_error(iscsi, "Failed to send EXTENDED COPY"
+				" command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_scsi_command_sync(struct iscsi_context *iscsi, int lun,
 			struct scsi_task *task, struct iscsi_data *data)
@@ -1818,11 +1868,9 @@ iscsi_discovery_cb(struct iscsi_context *iscsi _U_, int status,
                 }
         }
 
-	if (state != NULL) {
-		state->status    = status;
-		state->finished = 1;
-                state->ptr = dahead;
-	}
+	state->status    = status;
+	state->finished = 1;
+	state->ptr = dahead;
 }
 
 struct iscsi_discovery_address *
