@@ -320,7 +320,7 @@ do {                                                                        \
         } while (0);
 
 #define READ10(...)                                                     \
-        do {                                                            \
+        ({								\
                 int _r;                                                 \
                 _r = read10(__VA_ARGS__);                               \
                 if (_r == -2) {                                         \
@@ -331,7 +331,8 @@ do {                                                                        \
                         return;                                         \
                 }                                                       \
                 CU_ASSERT_EQUAL(_r, 0);                                 \
-        } while (0);
+		_r;							\
+        })
 
 #define READ12(...)                                                     \
         do {                                                            \
@@ -418,7 +419,7 @@ do {                                                                        \
         } while (0);
 
 #define RECEIVE_COPY_RESULTS(...)                                       \
-        do {                                                            \
+        ({								\
                 int _r;                                                 \
                 _r = receive_copy_results(__VA_ARGS__);                 \
                 if (_r == -2) {                                         \
@@ -429,7 +430,8 @@ do {                                                                        \
                         return;                                         \
                 }                                                       \
                 CU_ASSERT_EQUAL(_r, 0);                                 \
-        } while (0);
+		_r;							\
+        })
 
 #define RELEASE6(...)                                                   \
         do {                                                            \
@@ -601,7 +603,7 @@ do {                                                                        \
         } while (0);
 
 #define WRITE10(...)                                                    \
-        do {                                                            \
+        ({								\
                 int _r;                                                 \
                 _r = write10(__VA_ARGS__);                              \
                 if (_r == -2) {                                         \
@@ -612,7 +614,8 @@ do {                                                                        \
                         return;                                         \
                 }                                                       \
                 CU_ASSERT_EQUAL(_r, 0);                                 \
-        } while (0);
+		_r;							\
+        })
 
 #define WRITE12(...)                                                    \
         do {                                                            \
@@ -819,7 +822,8 @@ int all_zero(const unsigned char *buf, unsigned size);
 int prin_task(struct scsi_device *sdev, int service_action,
     int success_expected);
 int prin_read_keys(struct scsi_device *sdev, struct scsi_task **tp,
-    struct scsi_persistent_reserve_in_read_keys **rkp);
+                   struct scsi_persistent_reserve_in_read_keys **rkp,
+                   uint16_t allocation_len);
 int prout_register_and_ignore(struct scsi_device *sdev,
     unsigned long long key);
 int prout_register_key(struct scsi_device *sdev,
