@@ -1,4 +1,3 @@
-/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /*
    iscsi-test tool
 
@@ -158,6 +157,7 @@ static CU_TestInfo tests_preventallow[] = {
 
 static CU_TestInfo tests_prin_read_keys[] = {
         { (char *)"Simple", test_prin_read_keys_simple },
+        { (char *)"Truncate", test_prin_read_keys_truncate },
         CU_TEST_INFO_NULL
 };
 
@@ -769,6 +769,37 @@ print_usage(void)
         fprintf(stderr, "\n");
 }
 
+/* Clear persistent reservations and reservation keys left by a previous run */
+static int clear_pr(struct scsi_device *sdev)
+{
+        int i, res;
+        struct scsi_task *pr_task;
+        struct scsi_persistent_reserve_in_read_keys *rk;
+
+        res = 0;
+        if (prin_read_keys(sdev, &pr_task, &rk, 16384) != 0)
+                goto out;
+
+        res = -1;
+        if (rk->num_keys && data_loss == 0)
+                goto out;
+
+        res = 0;
+        for (i = 0; i < rk->num_keys; i++) {
+                res = prout_register_and_ignore(sdev, rk->keys[i]);
+                if (res)
+                        break;
+                res = prout_register_key(sdev, 0, rk->keys[i]);
+                if (res)
+                        break;
+        }
+
+        scsi_free_scsi_task(pr_task);
+
+out:
+        return res;
+}
+
 void
 test_setup(void)
 {
@@ -829,6 +860,7 @@ suite_cleanup(void)
         for (i = 0; i < mp_num_sds; i++) {
                 if (mp_sds[i]->iscsi_url) {
                         if (mp_sds[i]->iscsi_ctx) {
+                                clear_pr(mp_sds[i]);
                                 iscsi_logout_sync(mp_sds[i]->iscsi_ctx);
                                 iscsi_destroy_context(mp_sds[i]->iscsi_ctx);
                                 mp_sds[i]->iscsi_ctx = NULL;
@@ -1061,33 +1093,6 @@ static void free_scsi_device(struct scsi_device *sdev)
         free(sdev);
 }
 
-/* Clear persistent reservations and reservation keys left by a previous run */
-static int clear_pr(struct scsi_device *sdev)
-{
-        int i, res;
-        struct scsi_task *pr_task;
-        struct scsi_persistent_reserve_in_read_keys *rk;
-
-        res = 0;
-        if (prin_read_keys(sdev, &pr_task, &rk) != 0)
-                goto out;
-
-        res = -1;
-        if (rk->num_keys && data_loss == 0)
-                goto out;
-
-        res = 0;
-        for (i = 0; i < rk->num_keys; i++) {
-                prout_register_and_ignore(sdev, rk->keys[i]);
-                prout_register_key(sdev, 0, rk->keys[i]);
-        }
-
-        scsi_free_scsi_task(pr_task);
-
-out:
-        return res;
-}
-
 int
 main(int argc, char *argv[])
 {
@@ -1198,7 +1203,8 @@ main(int argc, char *argv[])
                 memset(mp_sds[mp_num_sds], '\0', sizeof(struct scsi_device));
                 mp_sds[mp_num_sds]->sgio_fd = -1;
 
-                if (!strncmp(argv[optind], "iscsi://", 8)) {
+                if (!strncmp(argv[optind], "iscsi://", 8) ||
+                    !strncmp(argv[optind], "iser://", 7)) {
                         mp_sds[mp_num_sds]->iscsi_url = strdup(argv[optind++]);
 #ifdef HAVE_SG_IO
                 } else {
@@ -1241,6 +1247,11 @@ main(int argc, char *argv[])
                 }
         }
 
+	if (clear_pr(sd) < 0) {
+		printf("One or more persistent reservations keys have been registered\n");
+		goto err_sds_free;
+	}
+
         if (mp_num_sds > 1) {
                 /* check that all multipath sds identify as the same LU */
                 res = mpath_check_matching_ids(mp_num_sds, mp_sds);
@@ -1437,11 +1448,6 @@ main(int argc, char *argv[])
         }
         scsi_free_scsi_task(task);
 
-        if (clear_pr(sd) < 0) {
-                printf("One or more persistent reservations keys have been registered\n");
-                return -1;
-        }
-
         /* BLKSECTGET for /dev/sg* is a shitshow under linux.
          * Even 4.2 kernels return number of bytes instead of number
          * of sectors here. Just force it to 120k and let us get on with
