@@ -1,4 +1,3 @@
-/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /* 
    Copyright (C) 2013 by Lee Duncan <lee@gonzoleeman.net>
    
@@ -37,9 +36,8 @@ test_prin_read_keys_simple(void)
         logging(LOG_VERBOSE, LOG_BLANK_LINE);
         logging(LOG_VERBOSE, "Test Persistent Reserve IN READ_KEYS works.");
 
-        ret = prin_read_keys(sd, &task, NULL);
+        ret = prin_read_keys(sd, &task, NULL, 16384);
         if (ret == -2) {
-                logging(LOG_NORMAL, "[SKIPPED] PERSISTEN RESERVE IN is not implemented.");
                 CU_PASS("PERSISTENT RESERVE IN is not implemented.");
                 return;
         }        
