@@ -1,4 +1,3 @@
-/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /* 
    Copyright (C) 2013 by Lee Duncan <leeman.duncan@gmail.com>
 
@@ -47,7 +46,6 @@ verify_persistent_reserve_ownership(struct scsi_device *sd1, struct scsi_device
         /* register our reservation key with the target */
         ret = prout_register_and_ignore(sd1, key1);
         if (ret == -2) {
-                logging(LOG_NORMAL, "[SKIPPED] PERSISTEN RESERVE OUT is not implemented.");
                 CU_PASS("PERSISTENT RESERVE OUT is not implemented.");
                 return;
         }        
