@@ -1,6 +1,5 @@
-/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /* 
-   Copyright (C) 2012 by Lee Duncan <lee@gonzoleeman.net>
+   Copyright (C) 2013 Ronnie Sahlberg <ronniesahlberg@gmail.com>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,28 +24,25 @@
 #include "iscsi-test-cu.h"
 
 void
-test_read10_0blocks(void)
+test_read16_0blocks(void)
 {
-        logging(LOG_VERBOSE, LOG_BLANK_LINE);
-        logging(LOG_VERBOSE, "Test READ10 0-blocks at LBA==0");
+        CHECK_FOR_SBC;
 
-        READ10(sd, NULL, 0, 0, block_size, 0, 0, 0, 0, 0, NULL,
+        logging(LOG_VERBOSE, LOG_BLANK_LINE);
+        logging(LOG_VERBOSE, "Test READ16 0-blocks at LBA==0");
+        READ16(sd, NULL, 0, 0, block_size, 0, 0, 0, 0, 0, NULL,
                EXPECT_STATUS_GOOD);
 
-        if (num_blocks > 0x80000000) {
-                CU_PASS("[SKIPPED] LUN is too big");
-                return;
-        }
-
-        logging(LOG_VERBOSE, "Test READ10 0-blocks one block past end-of-LUN");
-        READ10(sd, NULL, num_blocks + 1, 0, block_size, 0, 0, 0, 0, 0, NULL,
+        logging(LOG_VERBOSE, "Test READ16 0-blocks one block past end-of-LUN");
+        READ16(sd, NULL, num_blocks + 1, 0, block_size, 0, 0, 0, 0, 0, NULL,
                EXPECT_LBA_OOB);
 
-        logging(LOG_VERBOSE, "Test READ10 0-blocks at LBA==2^31");
-        READ10(sd, NULL, 0x80000000, 0, block_size, 0, 0, 0, 0, 0, NULL,
+        logging(LOG_VERBOSE, "Test READ16 0-blocks at LBA==2^63");
+        READ16(sd, NULL, 0x8000000000000000ULL, 0,
+               block_size, 0, 0, 0, 0, 0, NULL,
                EXPECT_LBA_OOB);
 
-        logging(LOG_VERBOSE, "Test READ10 0-blocks at LBA==-1");
-        READ10(sd, NULL, -1, 0, block_size, 0, 0, 0, 0, 0, NULL,
+        logging(LOG_VERBOSE, "Test READ16 0-blocks at LBA==-1");
+        READ16(sd, NULL, -1, 0, block_size, 0, 0, 0, 0, 0, NULL,
                EXPECT_LBA_OOB);
 }
