@@ -28,20 +28,10 @@
 void
 test_reserve6_simple(void)
 {
-	int ret;
+        logging(LOG_VERBOSE, LOG_BLANK_LINE);
+        logging(LOG_VERBOSE, "Test basic RESERVE6/RELEASE6 commands if supported");
 
+        RESERVE6(sd);
 
-	logging(LOG_VERBOSE, LOG_BLANK_LINE);
-	logging(LOG_VERBOSE, "Test basic RESERVE6/RELEASE6 commands if supported");
-
-	ret = reserve6(iscsic, tgt_lun);
-	if (ret == -2) {
-		logging(LOG_VERBOSE, "[SKIPPED] Target does not support RESERVE6. Skipping test");
-		CU_PASS("[SKIPPED] Target does not support RESERVE6. Skipping test");
-		return;
-	}
-	CU_ASSERT_EQUAL(ret, 0);
-
-	ret = release6(iscsic, tgt_lun);
-	CU_ASSERT_EQUAL(ret, 0);
+        RELEASE6(sd);
 }
