@@ -1,4 +1,3 @@
-/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /* 
    Copyright (C) 2013 Ronnie Sahlberg <ronniesahlberg@gmail.com>
    
@@ -25,20 +24,20 @@
 #include "iscsi-test-cu.h"
 
 void
-test_verify12_0blocks(void)
+test_verify16_0blocks(void)
 {
         logging(LOG_VERBOSE, LOG_BLANK_LINE);
-        logging(LOG_VERBOSE, "Test VERIFY12 0-blocks at LBA==0");
+        logging(LOG_VERBOSE, "Test VERIFY16 0-blocks at LBA==0");
 
-        logging(LOG_VERBOSE, "Test VERIFY12 0-blocks one block past end-of-LUN");
-        VERIFY12(sd, num_blocks + 1, 0, block_size, 0, 0, 1, NULL,
+        logging(LOG_VERBOSE, "Test VERIFY16 0-blocks one block past end-of-LUN");
+        VERIFY16(sd, num_blocks + 1, 0, block_size, 0, 0, 1, NULL,
                  EXPECT_LBA_OOB);
 
-        logging(LOG_VERBOSE, "Test VERIFY12 0-blocks at LBA==2^31");
-        VERIFY12(sd, 0x80000000, 0, block_size, 0, 0, 1, NULL,
+        logging(LOG_VERBOSE, "Test VERIFY16 0-blocks at LBA==2^63");
+        VERIFY16(sd, 0x8000000000000000ULL, 0, block_size, 0, 0, 1, NULL,
                  EXPECT_LBA_OOB);
 
-        logging(LOG_VERBOSE, "Test VERIFY12 0-blocks at LBA==-1");
-        VERIFY12(sd, -1, 0, block_size, 0, 0, 1, NULL,
+        logging(LOG_VERBOSE, "Test VERIFY16 0-blocks at LBA==-1");
+        VERIFY16(sd, -1, 0, block_size, 0, 0, 1, NULL,
                  EXPECT_LBA_OOB);
 }
