@@ -1,4 +1,3 @@
-/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /*
    Copyright (C) 2013 by Ronnie Sahlberg <ronniesahlberg@gmail.com>
 
@@ -62,6 +61,7 @@ test_writeverify10_residuals(void)
 
         memset(task, 0, sizeof(struct scsi_task));
         task->cdb[0] = SCSI_OPCODE_WRITE_VERIFY10;
+        task->cdb[1] = 2; /* BYTCHK = 1 */
         task->cdb[8] = 1;
         task->cdb_size = 10;
         task->xfer_dir = SCSI_XFER_WRITE;
@@ -121,6 +121,7 @@ test_writeverify10_residuals(void)
 
         memset(task, 0, sizeof(struct scsi_task));
         task->cdb[0] = SCSI_OPCODE_WRITE_VERIFY10;
+        task->cdb[1] = 2; /* BYTCHK = 1 */
         task->cdb[8] = 1;
         task->cdb_size = 10;
         task->xfer_dir = SCSI_XFER_WRITE;
@@ -164,6 +165,7 @@ test_writeverify10_residuals(void)
 
         memset(task, 0, sizeof(struct scsi_task));
         task->cdb[0] = SCSI_OPCODE_WRITE_VERIFY10;
+        task->cdb[1] = 2; /* BYTCHK = 1 */
         task->cdb[8] = 1;
         task->cdb_size = 10;
         task->xfer_dir = SCSI_XFER_WRITE;
@@ -213,6 +215,7 @@ test_writeverify10_residuals(void)
 
         memset(task, 0, sizeof(struct scsi_task));
         task->cdb[0] = SCSI_OPCODE_WRITE_VERIFY10;
+        task->cdb[1] = 2; /* BYTCHK = 1 */
         task->cdb[8] = 2;
         task->cdb_size = 10;
         task->xfer_dir = SCSI_XFER_WRITE;
@@ -266,6 +269,7 @@ test_writeverify10_residuals(void)
 
         memset(task, 0, sizeof(struct scsi_task));
         task->cdb[0] = SCSI_OPCODE_WRITE_VERIFY10;
+        task->cdb[1] = 2; /* BYTCHK = 1 */
         task->cdb[8] = 1;
         task->cdb_size = 10;
         task->xfer_dir = SCSI_XFER_WRITE;
@@ -338,6 +342,7 @@ test_writeverify10_residuals(void)
 
         memset(task, 0, sizeof(struct scsi_task));
         task->cdb[0] = SCSI_OPCODE_WRITE_VERIFY10;
+        task->cdb[1] = 2; /* BYTCHK = 1 */
         task->cdb[8] = 2;
         task->cdb_size = 10;
         task->xfer_dir = SCSI_XFER_WRITE;
