@@ -1,4 +1,3 @@
-/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /* 
    Copyright (C) 2014-2015 by Peter Lieven <pl@kamp.de>
    
@@ -186,7 +185,7 @@ out:
 
 void fill_read_queue(struct client *client)
 {
-	int num_blocks;
+	int64_t num_blocks;
 
 	if (finished) return;
 
@@ -210,7 +209,7 @@ void fill_read_queue(struct client *client)
 
 		task = iscsi_read16_task(client->iscsi,
 								client->lun, client->pos,
-								num_blocks * client->blocksize,
+								(uint32_t)(num_blocks * client->blocksize),
 								client->blocksize, 0, 0, 0, 0, 0,
 								cb, client);
 		if (task == NULL) {
